import time, random

class ChatUserTracker:
    "Chat User Tracker class"

    # Constructor
    def __init__(self):
        self.content_id = "0" 
        self.alive_time = 15 # in seconds
        self.tracks = []
        self.browsers = {}
        self.members = []
    
    # Private methods
    def updateTracks(self, user_changed = 0):
        "update and delete old tracks"
        index = 0
        b_continue = 1
        current_time = int(time.time())
        
        while b_continue and index < len(self.tracks):
            if ((self.tracks[index].get('date') + self.alive_time) < current_time): 
                browser_id = self.tracks[index].get('browser_id')
                if (self.browsers.get(browser_id) is not None):
                    if ((self.browsers[browser_id].get('date') + self.alive_time) < current_time):
                        user_changed = 1
                        del self.browsers[browser_id]
                index = index + 1
            else:
                b_continue = 0
                
                
        if (index > 0):
            del self.tracks[0:index]
            
            
        #  Update members list
        if user_changed == 1: 
            self.updateMembers()
    
    def updateMembers(self):
        self.members = []
        index = 0
        for browser in self.browsers.values():
            user_name = browser.get('user_name')
            if user_name not in self.members:
                self.members.append(user_name)        
        
        # Sort list and save modifications  
        self.members.sort()
        
        # Save data and generate new content_id
        self.content_id = str(int(time.time())) + str(random.randint(10000,99999))
    
    # Public methods
    def track(self, request):
        user_changed = 0
        user_name = request['AUTHENTICATED_USER'].getUserName()
        if user_name != 'Anonymous User':
            date = int(time.time())
            browser_id = request.SESSION.getBrowserIdManager().getBrowserId()
            
            # Add a new track in tracks list
            self.tracks.append({'date' : date, 'browser_id' : browser_id}) 
            
            # Update or add a new browser_id key in browsers dictionnary
            if user_name not in self.members:
                user_changed = 1
            elif self.browsers.get(browser_id) is not None:
                if self.browsers[browser_id].get('user_name') != user_name:
                    user_changed = 1
                
            self.browsers[browser_id] = {'user_name' : user_name, 'date' : date}
                   
        # Update tracks     
        self.updateTracks(user_changed)

    def getContentId(self):
        "Get content id"
        return self.content_id
    
    def getMembers(self):
        "Get list of connected members"
        return self.members
        
    def setAliveTime(self, alive_time):
        "Set alive_time attrtibute"
        self.alive_time = alive_time
