#################################################################################
#                                                                               #
#                       copyright (c) 2003 Ingeniweb SARL                       #
#                                                                               #
#################################################################################

"""
PloneChat main class
"""


# Python imports
import string, base64, StringIO, random, time, cPickle

# CMF imports
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

# Zope imports
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Globals import InitializeClass

# PloneChat imports
from global_symbols import *
from ChatUserTracker import ChatUserTracker

# PloneChat definition for Plone 1
factory_type_information = {
  'id'             : 'PloneChat',
  'title'          : 'Chat',
  'portal_type'    : 'PloneChat',
  'meta_type'      : 'PloneChat',
  'description'    : 'A chat type',
  'content_icon'   : 'PloneChat.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'PloneChat',
  'factory'        : 'addPloneChat',
  'immediate_view' : 'PloneChat_edit',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'PloneChat_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'portal_form/PloneChat_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'moderation',
    'name'          : 'Moderation',
    'action'        : 'PloneChat_moderationForm',
    'permissions'   : (PloneChat_moderatePermission, ),
    'category'      : 'object',
    },
    {
    'id'            : 'members',
    'name'          : 'Chat members',
    'action'        : 'PloneChat_membersForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    ),
  }

# PloneChat definition for Plone 2
factory_type_information2 = {
  'id'             : 'PloneChat',
  'portal_type'    : 'PloneChat',
  'meta_type'      : 'PloneChat',
  'description'    : 'A chat type',
  'content_icon'   : 'PloneChat.gif',                          # Icon has to be acquired from a skin folder
  'product'        : 'PloneChat',
  'factory'        : 'addPloneChat',
  'immediate_view' : 'PloneChat_edit2',
  'filter_content_types' : 0,                                   # This is for folderish types
  'allowed_content_types': (),                                  # If filter_content_types is true, this field can hold allowed meta_types
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${object_url}/PloneChat_view2',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'string:${object_url}/PloneChat_editForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'moderation',
    'name'          : 'Moderation',
    'action'        : 'string:${object_url}/PloneChat_moderationForm2',
    'permissions'   : (PloneChat_moderatePermission, ),
    'category'      : 'object',
    },
    {
    'id'            : 'members',
    'name'          : 'Chat members',
    'action'        : 'string:${object_url}/PloneChat_membersForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'string:${object_url}/metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'content_status_history',
    'name'          : 'State',
    'action'        : 'string:${object_url}/content_status_history',
    'condition'     : 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())',
    'permissions'   : (CMFCorePermissions.View,),
    'category'      : 'object_tabs',
    'visible'       : 0
    },
    ),
  }  
# The usual factory
def addPloneChat(self, id, title = '', REQUEST = {}):
    """
    Factory method for a PloneChat object
    """
    obj = PloneChat(id, title)
    self._setObject(id, obj)
    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')



class PloneChat(PortalContent, DefaultDublinCoreImpl):
    """PloneChat class"""

    #                                                                           #
    #                            BASIC CLASS BEHAVIOUR                          #
    #                                                                           #

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    meta_type = factory_type_information['meta_type']                   # Has not to be the same as FTI but it is the most usual case.

    manage_options = PortalContent.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)            # $$$ Is this clever ? Isn't it better to make the object private ?
                                                                        # This method makes all class properties & methods protected by View by default

    # Dummy permissions
    security.declareProtected(PloneChat_editPermission,"dummy_edit_perm",)
    security.declareProtected(PloneChat_unrestrictedChatPermission, "dummy_uchat_perm", )
    security.declareProtected(PloneChat_moderatePermission, "dummy_moderate_perm", )
    security.declareProtected(PloneChat_restrictedChatPermission, "dummy_rchat_perm", )



    # Init method
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        PloneChat.inheritedAttribute('__init__')(self)
        self.id = id
        self.title = title
        self.chat_size = 100                            # Max chat size
        self.pending_questions_list = []                # $$$ Should be in a file instead of beeing persistant and historized
        self._seed = str(int(time.time())) + str(random.randint(10000,99999))
        self.moderation = 0                             # Moderation status

        self.chat_date = DateTime()                     # Scheduled date of the chat
        self.refresh_rate = 10                          # Maximum refresh rate of the chat
        self.time_format = "%H:%M:%S"                   # Message header time format 
        
    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        # We set the object creator as a Reviewer - as it won't be possible to do so later because of permission restrictions
        obj = self
        mtool = getToolByName(self, 'portal_membership')
        member_id = mtool.getAuthenticatedMember().getUserName()
        roles = list(obj.get_local_roles_for_userid( userid=member_id ))
        if "Reviewer" not in roles:
            roles.append( "Reviewer" )
            obj.manage_setLocalRoles( member_id, roles )
        Log(LOG_DEBUG, "local roles", self.get_local_roles_for_userid(member_id))

        # We reindex the object
        self.indexObject()



    #                                                                           #
    #                           CONTENT EDITING METHODS                         #
    #                                                                           #



    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(PloneChat_editPermission, 'manage_editPloneChat')
    def manage_editPloneChat(self, moderation = 0, refresh_rate = 10, time_format='', chatDay=None, chatMonth=None, chatYear=None, REQUEST=None):
        """
        manage_editPloneChat(self, moderation = 0, refresh_rate = 10, REQUEST = {}) => object modification method
        """
        # Attributes
        self.moderation = moderation
        self.refresh_rate = refresh_rate
        self.time_format = time_format
        
        if chatDay and chatMonth and chatYear:
            chat_date_string = '%s/%s/%s' % (chatDay, chatMonth, chatYear)
            self.chat_date = DateTime(chat_date_string)

        # Reindex
        self.reindexObject()

        # Change user_tracker properties
        chat_user_tracker = self.getPloneChatData("user_tracker", ChatUserTracker())
        chat_user_tracker.setAliveTime(refresh_rate + 5)
        self.setPloneChatData("user_tracker", chat_user_tracker)
        
        # Redirect
        if REQUEST is not None:
            return self.folder_contents(self, REQUEST, portal_status_message='Updated content.')
        

    #                                                                           #
    #                             CMF CATALOG SUPPORT                           #
    #                                                                           #


    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        return "%s %s %s" % (self.Title(), self.Description(), string.join(map(lambda x: x['text'], self.getChat(), )))



    #                                                                           #
    #                         QUESTION PROPOSAL SUPPORT                         #
    #                                                                           #


    security.declareProtected(PloneChat_restrictedChatPermission, "postModerated")
    def postModerated(self, question, username = ""):
        """
        postQuestion(self, question, username = "") => Add a new question to the moderator's list.
        The new question will be forwarded to the guest if only if it is moderated.
        If username is None, then the AUTH_USER name is taken.
        """
        if not username:
            mtool = getToolByName(self, 'portal_membership')
            username = mtool.getAuthenticatedMember().getUserName()
        self.pending_questions_list.append(
            {
            'timestamp': DateTime(),
            'user': username,
            'question': question,
            'id': random.randint(1000,9999),
            })
        self.pending_questions_list = self.pending_questions_list               # Ensure persistancy


    security.declareProtected(PloneChat_moderatePermission, "listPendingQuestions")
    def listPendingQuestions(self,):
        """
        listPendingQuestions(self,) => Return a list of questions that has to be moderated.
        A question is a dict having at least 'timestamp', 'user' and 'question' keys
        """
        return self.pending_questions_list

    #                                                                           #
    #                           STRAIGHT POSTING SUPPORT                        #
    #                                                                           #

    security.declareProtected(CMFCorePermissions.View, "postReply")
    def postReply(self, question, username = ""):
        """
        postReply(self, question, username = "") => Check permissions to call either postModerated or postUnmoderated methods.

        We set it as a View permission so that everyone can call it.
        """
        # Test posting ability
        if not self.canPost():
            return

        # Test moderation ability
        if self.canChat():
            return self.postUnmoderated(question, username)
        return self.postModerated(question, username)
        

    security.declareProtected(PloneChat_unrestrictedChatPermission, "postUnmoderated")
    def postUnmoderated(self, question, username = ""):
        """
        postUnmoderated(self, question, username = "") => Same as postModerated without moderation
        """
        if not username:
            mtool = getToolByName(self, 'portal_membership')
            username = mtool.getAuthenticatedMember().getUserName()
        self._addLine(
            {
            'timestamp': DateTime(),
            'user': username,
            'text': question,
            })
    

    #                                                                           #
    #                           MODERATION SUPPORT                              #
    #                                                                           #

    security.declareProtected(CMFCorePermissions.View, "canPost")
    def canPost(self,):
        """
        canPost(self,) => Return 1 if the user can post something on the chat
        """
        mtool = getToolByName(self, 'portal_membership')
        try:            # $$$ Have to find a fastest way to do so
            return mtool.checkPermission(PloneChat_unrestrictedChatPermission, self) or mtool.checkPermission(PloneChat_restrictedChatPermission, self)
        except:
            return 0


    security.declareProtected(CMFCorePermissions.View, "canChat")
    def canChat(self,):
        """
        canChat(self,) => return 0 if the post will be moderated or post is not allowed, 1 if post won't be moderated
        """
        mtool=getToolByName(self, 'portal_membership')
        if not self.moderation:
            return 1
        try:            # $$$ Have to find a fastest way to do so
            return mtool.checkPermission(PloneChat_unrestrictedChatPermission, self)
        except:
            return 0
        

    security.declareProtected(PloneChat_moderatePermission, "acceptQuestion")
    def acceptQuestion(self, id, text = None):
        """
        acceptQuestion(self, id, text = None) => Transmit a question to the guest(s)
        The question is then readable by all

        if text is provided, the question's text is replaced by the supplied text.
        """
        Log(LOG_DEBUG, "Accepting question", id, self.pending_questions_list)
        index = 0
        for q in self.pending_questions_list:
            if q['id'] == id:
                q = self.pending_questions_list[index]
                del self.pending_questions_list[index]
                if not text:
                    text = q['question']
                q['text'] = text
                self._addLine(q)
                break
            index += 1
        self.pending_questions_list = self.pending_questions_list               # Ensure persistancy



    security.declareProtected(PloneChat_moderatePermission, "ignoreQuestion")
    def ignoreQuestion(self, id):
        """
        ignoreQuestion(self, id) => Drop a proposed question
        """
        index = 0
        for q in self.pending_questions_list:
            if q['id'] == id:
                del self.pending_questions_list[index]
                break
            index += 1
        
        self.pending_questions_list = self.pending_questions_list               # Ensure persistancy



    #                                                                           #
    #                             ACTUAL CHAT SUPPORT                           #
    #                                                                           #


    security.declareProtected(CMFCorePermissions.View, "getChat")
    def getChat(self, size = 0, reverse = 0):
        """
        getChat(self, size = 0, reverse = 0) => list of dicts

        if reverse is 0, return first chat entry first. Else, return last chat entry first.

        Return the chat content as a list of dictionaries having 'user', 'text' and 'timestamp' keys. The latter one is a DateTime() object
        """
        username = None
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        if member:
            username = member.getUserName()

        # Return chat lines
        ret = []
        for line in self._readLines():
            (timestamp, user, text) = string.split(string.strip(line), ';')
            dt = DateTime(timestamp)
            current = not cmp(user, username)
            header = "[%s] %s: " % (dt.strftime(self.time_format), user, )
            ret.append(
                {"timestamp": dt,
                 "prettytime": dt.strftime(r"%H:%M:%S"),
                 "user": user,
                 "text": base64.decodestring(text),
                 "current": current,                            # current is true if it is a current user's post
                 "header": header,                              # [hh:mm:ss] user:
                 })
        ret = ret[-size:]
   
        if reverse:
            ret.reverse()
        return ret


    #                                                                           #
    #                         UNDERLYING CHATFILE SUPPORT                       #
    #                                                                           #


    security.declarePrivate("_getChatFileName")
    def _getChatFileName(self,):
        """
        _getChatFileName(self,) => Return a chat filename
        """
        ret = os.path.join(
            var_dir,
            "chat_%s" % (self._seed),
            )
        Log(LOG_DEBUG, "Chat file name", ret)
        return ret
    

    security.declarePrivate("_addLine")
    def _addLine(self, line_info):
        """
        _addLine(self, line_info) => add a line into the chat file

        line_info is a dict having 'user', 'text' and 'timestamp' keys (the latter being a DateTime() object)
        """
        text = string.strip(line_info['text'])
        if not text:
            return      # Ignore empty lines
        line = string.join(
            (str(line_info['timestamp']),
             line_info['user'],
             string.join(string.split(base64.encodestring(text)[:-1]), " "),              # We ignore linebreaks
             ),
            ";",
            ) + "\n"
        Log(LOG_DEBUG, "Writing line", line)
        f = open(self._getChatFileName(), "a+b")
        try:
            f.write(line)
        finally:
            f.close()
        self.setPloneChatData("chat_content_id", str(int(time.time())) + str(random.randint(10000,99999)))

    
    security.declarePrivate("_readLines")
    def _readLines(self,):
        """
        _readLines(self,) => Return the file content as a list of lines.
        $$$ This is very memory-consuming for laaaaaarge chats.
        In the next version we should use buffers instead.
        """
        ret = []
        # We try to open the file, and return [] if we cannot (that is, if the file doesn't exist)
        try:
            f = open(self._getChatFileName(), "rb")
        except:
            return []
            
        try:
            l = string.strip(f.readline())
            while l:
                ret.append(l)
                l = string.strip(f.readline())
        finally:
            f.close()
        Log(LOG_DEBUG, "_readLines returns", ret)
        return ret


    # Clear all data in chat
    security.declareProtected(PloneChat_editPermission, 'clearChatContent')
    def clearChatContent(self, REQUEST=None):
        """
        clearChatContent(self,) => clear content of chat
        """
        Log(LOG_DEBUG, "clear chat content")
        f = open(self._getChatFileName(), "w+b")
        f.close()
            
        self.setPloneChatData("chat_content_id", str(int(time.time())) + str(random.randint(10000,99999)))
        
        # Redirect
        if REQUEST is not None:
            if REQUEST.has_key('RESPONSE'):
                return REQUEST.RESPONSE.redirect(self.absolute_url() + '/view?portal_status_message=Content cleared.')
    

    #                                                                           #
    #                         UNDERLYING CHATFILE SUPPORT                       #
    #                                                                           #

    security.declareProtected(CMFCorePermissions.View, "track")
    def track(self, request, REQUEST = None):
        "Track plone chat user"
        # Can't access this method in url
        if REQUEST is None:
            chat_user_tracker = self.getPloneChatData("user_tracker", ChatUserTracker())
            chat_user_tracker.track(request)
            self.setPloneChatData("user_tracker", chat_user_tracker)
    
    security.declareProtected(CMFCorePermissions.View, "getMembers")
    def getMembers(self):
        "Get plone chat connected members"
        chat_user_tracker = self.getPloneChatData("user_tracker", ChatUserTracker())
        return chat_user_tracker.getMembers()
        
    security.declareProtected(CMFCorePermissions.View, "getMembersContentId")
    def getMembersContentId(self):
        "Get plone chat members content id"
        chat_user_tracker = self.getPloneChatData("user_tracker", ChatUserTracker())
        return chat_user_tracker.getContentId()
        
    security.declareProtected(CMFCorePermissions.View, "getChatContentId")
    def getChatContentId(self):
        "Get plone chat content id"
        return self.getPloneChatData("chat_content_id", "")
        
    security.declarePrivate("getPloneChatData")
    def getPloneChatData(self, key, object_if_none = None):
        "Get plone chat data from key"
        file_name = self._getChatFileName() + "_tracker"
        object = object_if_none
        
        try:
            f = open(file_name, "r+b")
            try:
                data_dictionnary = cPickle.load(f)
            finally:
                f.close()
        except:
            data_dictionnary = {}
          
        
        if data_dictionnary.has_key(key):
            object = data_dictionnary[key]
          
        return object
        
    security.declarePrivate("setPloneChatData")
    def setPloneChatData(self, key, object):
        "Set plone chat data in key"
        file_name = self._getChatFileName() + "_tracker"
        
        try:
            f = open(file_name, "r+b")
            data_dictionnary = cPickle.load(f)
            f.seek(0)
            f.truncate(0)
        except:
            f = open(file_name, "w+b")
            data_dictionnary = {}
                     
        data_dictionnary[key] = object
              
        try:
            cPickle.dump(data_dictionnary, f)
        finally:
            f.close()
        

# Class instanciation
InitializeClass(PloneChat)


